Package: triangle
Title: Distribution Functions and Parameter Estimates for the Triangle
        Distribution
Version: 1.1.0
Authors@R: 
    person(given = "Rob",
           family = "Carnell",
           role = c("aut", "cre"),
           email = "bertcarnell@gmail.com",
           comment = c(ORCID = "0009-0009-0465-7564")
           )
Description: Provides the "r, q, p, and d" distribution functions for the triangle distribution.  Also includes maximum likelihood estimation of parameters.
License: GPL (>= 2)
URL: https://bertcarnell.github.io/triangle/,
        https://github.com/bertcarnell/triangle/
BugReports: https://github.com/bertcarnell/triangle/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.3)
Imports: stats4, methods, boot
Suggests: testthat, knitr, rmarkdown, MASS, Rmpfr
VignetteBuilder: knitr
Collate: 'cdfe.R' 'dtriangle.R' 'fit-plots.R' 'ltriangle.r'
        'mle-utils.R' 'mle.R' 'mom.R' 'ptriangle.r' 'qtriangle.R'
        'rtriangle.r'
NeedsCompilation: no
Packaged: 2026-04-27 01:25:01 UTC; bertc
Author: Rob Carnell [aut, cre] (ORCID: <https://orcid.org/0009-0009-0465-7564>)
Maintainer: Rob Carnell <bertcarnell@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 05:10:03 UTC
Built: R 4.5.2; ; 2026-04-27 09:33:00 UTC; unix
