# Generated by roxygen2: do not edit by hand

S3method(AIC,triangle_mle)
S3method(BIC,triangle_mle)
S3method(coef,triangle_mle)
S3method(confint,triangle_mle)
S3method(logLik,triangle_mle)
S3method(print,triangle_mle)
S3method(profile,triangle_mle)
S3method(summary,triangle_mle)
S3method(vcov,triangle_mle)
export(compare_triangle_fit)
export(dltriangle)
export(dtriangle)
export(pltriangle)
export(ptriangle)
export(qltriangle)
export(qqtriangle)
export(qtriangle)
export(rltriangle)
export(rtriangle)
export(standard_triangle_mle)
export(triangle_cdfe)
export(triangle_mle)
export(triangle_mom)
importClassesFrom(stats4,profile.mle)
importClassesFrom(stats4,summary.mle)
importFrom(boot,boot)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(methods,new)
importFrom(stats,BIC)
importFrom(stats,coef)
importFrom(stats,ecdf)
importFrom(stats,integrate)
importFrom(stats,nls)
importFrom(stats,nls.control)
importFrom(stats,optim)
importFrom(stats,plot.ecdf)
importFrom(stats,ppoints)
importFrom(stats,printCoefmat)
importFrom(stats,qchisq)
importFrom(stats,qqplot)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(stats4,coef)
importFrom(stats4,confint)
importFrom(stats4,show)
importFrom(stats4,vcov)
